import 'package:flutter/material.dart';

enum ScreenSize {
  unknown,
  xsmall,
  small,
  medium,
  large,
  xlarge,
}

class Screen {
  static late double xlargeWidth = 1400;
  static late double largeWidth = 1200;
  static late double mediumWidth = 1024;
  static late double smallWidth = 380;

  /// If [useSafeSize] is true, the real device size is returned.
  /// Otherwise the available size without System UI padding.
  /// This is used as default for instance of [Screen.of(context, {bool? useSafeSize})]
  static bool useSafeSize = false;
  // static late double xsmallWidth = 300;

  final BuildContext context;
  late final MediaQueryData mediaQueryData;
  late final bool _useSafeSize;

  Screen._(this.context, {bool? useSafeSize}) {
    _useSafeSize = useSafeSize ?? Screen.useSafeSize;
    mediaQueryData = MediaQuery.of(context);
  }

  factory Screen.of(BuildContext context, {bool? useSafeSize}) {
    return Screen._(context, useSafeSize: useSafeSize);
  }

  /// If [useSafeSize] is true, it returns [saveWidth], otherwise [screenWidth].
  /// Otherwise the available size without padding.
  double get width => _useSafeSize ? saveWidth : screenWidth;

  /// If [useSafeSize] is true, it returns [saveHeight], otherwise [screenHeight].
  /// Otherwise the available size without padding.
  double get height => _useSafeSize ? saveHeight : screenHeight;

  /// The width of the media in logical pixels.
  /// MediaQuery.of(context).size.width
  double get screenWidth => mediaQueryData.size.width;

  /// The height of the media in logical pixels.
  /// MediaQuery.of(context).size.height
  double get screenHeight => mediaQueryData.size.height;

  /// The width of the media in logical pixels without System UI padding.
  double get saveWidth => mediaQueryData.size.width - mediaQueryData.padding.left - mediaQueryData.padding.right;

  /// The height of the media in logical pixels without System UI padding.
  double get saveHeight => mediaQueryData.size.height - mediaQueryData.padding.top - mediaQueryData.padding.bottom;

  Orientation get orientation => mediaQueryData.orientation;
  bool get isPortrait => mediaQueryData.orientation == Orientation.portrait;
  bool get isLandscape => mediaQueryData.orientation == Orientation.landscape;

  bool isXL() => isExtraLargeScreen;
  bool get isExtraLargeScreen => width >= xlargeWidth;

  bool isL() => isLargeScreen;
  bool get isLargeScreen => width >= largeWidth && width < xlargeWidth;

  bool isM() => isMediumScreen;
  bool get isMediumScreen => width >= mediumWidth && width < largeWidth;

  bool isS() => isSmallScreen;
  bool get isSmallScreen => width >= smallWidth && width < mediumWidth;

  bool isXS() => isExtraSmallScreen;
  bool get isExtraSmallScreen => width < smallWidth;

  ScreenSize get screenSize => getScreenSize();
  ScreenSize getScreenSize() {
    if (isExtraSmallScreen) {
      return ScreenSize.xsmall;
    } else if (isSmallScreen) {
      return ScreenSize.small;
    } else if (isMediumScreen) {
      return ScreenSize.medium;
    } else if (isLargeScreen) {
      return ScreenSize.large;
    } else if (isExtraLargeScreen) {
      return ScreenSize.xlarge;
    } else {
      return ScreenSize.unknown;
    }
  }
}
